//*****************************************************//
//Dateiname: CompressFile.c
//Autor: Ptschner Stephan
//Projekt: niCE - nice insignificant compression engine
//Zweck: Modul, welches eine Datei mittels Huffmankomprimierung komprimiert
//Datum der letzten Aenderung: 11.05.2003
//*****************************************************//

//Eigene spezifische INCLUDES
#include "../nice.h"
#include "../huffman.h"
#include "../globals.h"

//***************************************************************************
//
//					Funktionen
//
//***************************************************************************

/**
 *
 *	@function: Hauptfunktion, die von Datei CompressFile.c zur Verfgung gestellt wird
 *				komprimiert eine bergebene Datei mit Hilfe einer bergebenen Kodiertabelle
 *	@params:
 *				pt_FileUncompressed ... Zeiger auf die unkomprimierte Datei aus welcher die Daten zum Komprimieren ausgelesen werden
 *				pt_FileCompressed ... Zeiger auf eine geffnete Datei, in welche die komprimierten Daten geschrieben werden sollen
 *				pt_array_CodingTable ... Zeiger auf eine Tabelle, die fr jeden Char eine Bitsequenz enthlt (der Char wird als diese Bitsequenz codiert)
 *				BufferBitSequence ... Buffer fr noch nicht in die Datei geschriebene Daten
 *				Continue ... != 0 so wird die Datei codiert, bei "0" wird nur die BufferSequenz in die Datei geschrieben
 *	@return value:
 *				KEINER
 *
 */
void CompressFile (FILE *pt_FileUncompressed, FILE *pt_FileCompressed, BitSequence *pt_array_CodingTable, BitSequence BufferBitSequence, char Continue)
{
	int ReadChar;				// Variable welche die ausgelesenen Bytes der unkomprimierten Datei aufnimmt
	unsigned char temp_length;	// Zwischenspeicher fr die Lnge der letzter letzten Buffersequenz (wird anschliessend als char ans Dateiende geschrieben)

	if (Continue)
	{
		while((ReadChar = fgetc(pt_FileUncompressed)) != EOF)	// Byteweises auslesen der (unkomprimierten) Ursprungsdatei bis zum Ende der Datei
			WriteBitSequenceToFile(pt_FileCompressed, &BufferBitSequence, (pt_array_CodingTable + ReadChar));	// Schreiben der Bitsequenz aus der Kodiertabelle die dem ausgelesenen Byte zugewiesen wurde
	}

	// Bitsequenzen sind geschrieben, jedoch bleiben im Buffer Bits die noch nicht geschrieben wurden
	temp_length = BufferBitSequence.length;
	
	{
		BufferBitSequence.length = sizeof(int) * 8;

			// Durch das Setzen der Lnge auf sizeof(int) * 8 wird die Funktion WriteBitSequenceToFile auf alle Flle den Buffer in die Datei schreiben
		WriteBitSequenceToFile(pt_FileCompressed, &BufferBitSequence, NULL);
	}

	// Am Ende der Datei wird die tatschliche Lnge der letzten Bitsequenz geschrieben, da diese andernfalls verloren ginge
	fputc(temp_length, pt_FileCompressed);

}

/**
 *
 *	@function: Hauptfunktion, die von Datei CompressFile.c zur Verfgung gestellt wird
 *				komprimiert eine bergebene Datei mit Hilfe einer bergebenen Kodiertabelle
 *	@params:
 *				pt_FileUncompressed ... Zeiger auf die unkomprimierte Datei aus welcher die Daten zum Komprimieren ausgelesen werden
 *				pt_FileCompressed ... Zeiger auf eine geffnete Datei, in welche die komprimierten Daten geschrieben werden sollen
 *				pt_array_CodingTable ... Zeiger auf eine Tabelle, die fr jeden Char eine Bitsequenz enthlt (der Char wird als diese Bitsequenz codiert)
 *				BufferBitSequence ... Buffer fr noch nicht in die Datei geschriebene Daten
 *				Continue ... != 0 so wird die Datei codiert, bei "0" wird nur die BufferSequenz in die Datei geschrieben
 *	@return value:
 *				1 ... Fehler (Datei konnte nicht geffnet werden)
 *				0 ... keine Fehler
 *
 */
int DecompressFile (char *file_source, char *file_target)
{
	FILE *pt_FileCompressed, *pt_FileUncompressed;
	Node *pt_RootNode = NULL;

	BitSequence BufferSequence;
	int temp_code = 0;
	BufferSequence.Code = &temp_code;
	BufferSequence.length = 0;

	if (((pt_FileCompressed = (fopen(file_source, "rb"))) == NULL) || ((pt_FileUncompressed = (fopen(file_target, "wb"))) == NULL))
	{
		temp_num_error++;
		return 1;
	}

	pt_RootNode = CreateTreeFromFile(pt_FileCompressed, &BufferSequence);

	{
		int BitFromFile = 0;
		Node *temp_Node = pt_RootNode;

		BitSequence ScoutSequence;
		int temp_code2 = 0;
		ScoutSequence.Code = &temp_code2;
		ScoutSequence.length = 0;

		GetBitsequenceFromFile(pt_FileCompressed, &ScoutSequence);

		while (ScoutSequence.length || BufferSequence.length)
		{
			
			if (BufferSequence.length == 1 && ScoutSequence.length)
			{
				BitFromFile = IsBitSetOfBS (BufferSequence, 1);
				*(BufferSequence.Code) = *(ScoutSequence.Code);
				BufferSequence.length = ScoutSequence.length;
				if (GetBitsequenceFromFile(pt_FileCompressed, &ScoutSequence) < 0)
				{
					BufferSequence.length = *(ScoutSequence.Code);
					*(ScoutSequence.Code) = 0;
					ScoutSequence.length = 0;
				}
			}
			else
				BitFromFile = GetBitFromFile(pt_FileCompressed, &BufferSequence);


			if(temp_Node->pt_Down[BitFromFile] != NULL)
				temp_Node = temp_Node->pt_Down[BitFromFile];

			if (temp_Node->iSimpleNode == LEAFNODE)
			{
				fputc(temp_Node->cContent, pt_FileUncompressed);
				temp_Node = pt_RootNode;
			}
		}
	}

	fclose(pt_FileCompressed);
	fclose(pt_FileUncompressed);
	return 0;

}