//*****************************************************//
//Dateiname: CreateCodingTable.c
//Autor: Ptschner Stephan
//Projekt: niCE - nice insignificant compression engine
//Zweck: Funktionen zum dynamischen Erzeugen einer Kodiertabelle und zum Lschen des dafr reservierten Platzes
//Datum der letzten Aenderung: 07.05.2003
//*****************************************************//

//Eigene spezifische INCLUDES
#include "../nice.h"
#include "../huffman.h"

//***************************************************************************
//
//					Funktionen
//
//***************************************************************************

/**
 *
 *	@function: Hauptfunktion, die von Datei CreateCodingTable.c zur Verfgung gestellt wird
 *				Erstellt aus einem Huffman-Baum eine Kodiertabelle, welche anschliessend zum Kodieren/Komprimieren der Datei benutzt werden kann
 *	@params:
 *				pt_RootNode ... Zeiger auf den Wurzelknoten eines binren Baumes, welcher einen Huffmanbaum beinhaltet
 *	@return value:
 *				ein Zeiger auf ein array aus 256 Bitsequenzelementen
 *
 */
BitSequence *CreateCodingTable (Node *pt_RootNode)
{
	BitSequence *pt_array_CodingTable;	// Zeiger, welcher anschliessend auf das Bitsequenz-Feld zeigen soll

	// dynamisches reservieren des Feldes
	pt_array_CodingTable = (BitSequence *) malloc (sizeof(BitSequence) * 256);
	// Initialisieren der Feldelemente des Bitsequenzfeldes
	memset(pt_array_CodingTable, 0, 256 * sizeof(BitSequence));

	// Durchlaufen des Baumes mittels TraverseTree
	// TraverseTree ruft bei jedem Knoten die Funktion "AddNodeToCodingTable" auf und bergibt dieser
	// sowohl einen Pointer auf den Knoten als auch einen Pointer auf die Kodiertabelle in welche diese dann den zugehrigen Huffman-Code schreibt
	TraverseTree(pt_RootNode, "", &AddNodeToCodingTable, pt_array_CodingTable);

	return pt_array_CodingTable;
}

/**
 *
 *	@function: Lscht den Platz der fr die Kodiertabelle dynamisch reserviert wurde
 *	@params:
 *				pt_array_CodingTable ... Zeiger das erste Element des Feldes der Kodiertabelle
 *	@return value:
 *				KEINER
 *
 */
void DestroyCodingTable (BitSequence *pt_array_CodingTable)
{
	// Lschen des dynamisch reservierten Platzes fr die Kodiertabelle
	free(pt_array_CodingTable);
}