//*****************************************************//
//Dateiname: huffman_analyse.c
//Autor: Walter Scherer
//Projekt: niCE - nice insignificant compression engine
//Zweck: Analyse der RLE-Datei, Berechnung der relativen Zeichenhaeufigkeit
//Datum der letzten Aenderung: 21.05.2003
//*****************************************************//

#include "../nice.h"
#include "../huffman.h"

//***************************************************************************
//
//					Funktionen
//
//***************************************************************************

/** 
* 
* @function: Berechnet die Auftrittswahrscheinlichkeit von Zeichen
* @params:	temp_counter ... Integer-Counter (Gesamtanzahl der Zeichen)
*			pt_array_probability ... Pointer auf Array mit Zeichenwahrscheinlichkeiten
* @return value: KEINER
*
*/ 

void calc_probability(int temp_counter, double *pt_array_probability) 
{
	int run_a;
	for (run_a=0;run_a<256;run_a++) 				//fuer jedes ASCII-Zeichen
	{
		
		pt_array_probability[run_a]=(pt_array_probability[run_a] / (double)temp_counter);		//errechne relative Haeufigkeit
	}
}
/** 
* 
* @function: Inkrementiert entsprechende (vom Parameter vorgegebene) Stelle im Array
* @params:			temp_character ... aktuelles Zeichen
*					pt_array_probability ... Array mit Zeichenwahrscheinlichkeiten
* @return value: KEINER
* 
*/
void count_char(int temp_character, double *pt_array_probability) 
{
	(*(pt_array_probability+temp_character))++;				//Inkrementiert die durch den Offset des jeweiligen Zeichens bezeichneten Speicherbereich des Arrays (dieser Speicherbereich entspricht genau dem bergebenen Zeichen)
}

/** 
* 
* @function: Analysiert RLE-Datei
* @params:			file ... Pointer auf RLE-Dateinamen
* @return value:	pt_array_probability ... Pointer auf das Array mit den Wahrscheinlichkeiten (fr das Aufstellen der Codetabelle fr Huffman-Codierung)
* 
*/
double *huffman_analyse_program(char *file)
{
	FILE *pt_file=NULL;
	int temp_character;											//Aktuelles Zeichen
	int temp_counter=0;											//Anzahl aller Zeichen
	int run_a;
	double *pt_array_probability=(double *)malloc(sizeof(double)*256);	//Array fr die Wharscheinlichkeiten

	for (run_a=0;run_a<256;run_a++)
		pt_array_probability[run_a]=0;

	if ((pt_file=fopen(file, "rb"))==NULL) 						//fals Datei nicht geffnet werden kann...
	{
		fprintf(stderr, "huffman_analyse: cannot open file\n");
		return NULL;
	}
	else {
		while ((temp_character=getc(pt_file))!=EOF)
		{
			temp_counter++;												//Jedes Zeichen vermerken (Gesamtzahl an Zeichen)
			count_char(temp_character, pt_array_probability);			//Abhaengig von seinem Wert Zeichen in Array eintragen
		}
	}
	calc_probability(temp_counter, pt_array_probability);			//Relative Haeufigkeiten ausrechnen
	fclose(pt_file);												//File-Pointer freigeben
	return pt_array_probability;
}