/*
File: rle_cod.c
Zweck: RLE Coding Routine fr das niCE Projekt.
Funktion: bernimmt als Argument den Namen der zu komprimierenden Datei sowie Generierung der komprimierten tmp-Datei und Rckgabe des Names
der tmp-Datei.
Author: Christian Sam / christian@niceshoes.cc
Datum: 11.05.2003

Hinweis: Teile dieser RLE Funktion beruhen auf der RLE Routine2 von David Bourgin / david.bourgin@ufrima.imag.fr
Teile dieses Programms setzen auerdem das Verstndniss der RLE Codierung vorraus. Siehe David Bourgin's codecs13.zip.
*/


/* Einbinden eines Standardheaders, bentigt werden mindestens: <stdio.h>, <memory.h>, <stdlib.h> */
#include "../nice.h"


/* Definition von Standard Error Codes */
#define NO_ERROR      0
#define BAD_FILE_NAME 1
#define BAD_ARGUMENT  2

/* Definition von Konstanten */
#define FALSE 0
#define TRUE  1

/* Definition von Globalen Variablen und Funktionen, welche fr das Handling und Einlesen mit geffneten Dateien bentigt werden 

!!! Wichtig, diesen Teil unbedingt auf Verstndnis durchlesen !!!

Es werden "end_of_data" und "read_byte" gemeinsam benutzt um Bytes einzulesen. Bytes werden in einer While Funktion eingelesen die solang 
ausgefhrt wird "while (!end_of_data())". Die Bytes werden in "val_byte_stored" gespeichert, als Flag das das Byte eingelesen wurde
wird "byte_stored_status" auf TRUE gesetzt. Im nchsten Durchgang wird in der Funktion "read_byte" bei gesetzem "byte_stored_status" Flag,
das eingelesen Byte von "val_byte_stored" auf unsigned char umgecastet und an die aufrufende Funktion bergeben, es wird auerdem das Flag
von "byte_stored_status" mittels FALSE wieder zurckgesetzt, soda im nchsten Durchgang der While Schleife wieder ein Byte eingelesen wird.
Die Bytes werden also solange an die aufrufende Funktion zurckgegeben, solang das Flag in der Funktion "end_of_date" gesetzt ist.

Die Funktionen "beginning_of_data" wird benutzt um an den Anfang des einzulesenden Filestreams zu kommen.
Die Funktion "write_byte" wird benutzt um den nun RLE komprimierten Stream in ein Ausgangsfile zurckzuschreiben. */

FILE *source_file,*dest_file;
int byte_stored_status=FALSE;
int val_byte_stored;
#define beginning_of_data()  (byte_stored_status=FALSE,(void)rewind(source_file))
#define end_of_data()  (byte_stored_status?FALSE:!(byte_stored_status=((val_byte_stored=fgetc(source_file))!=EOF)))
#define read_byte()  (byte_stored_status?byte_stored_status=FALSE,(unsigned char)val_byte_stored:(unsigned char)fgetc(source_file))
#define write_byte(byte)  ((void)fputc((byte),dest_file))


/* Um den Schreibvorgang zu verstehen ist ein Grundlagenwissen der RLE Codierung erforderlich. 
Siehe David Bourgin's COMPRESS.TXT > *** Second RLE scheme ***
Die Variablen der folgenden Funktion sind: "header_byte" = Escape Zeichen. "repeated_byte" = das zu codierende Zeichen. 
"repetition_number" = Anzahl der Wiederholungen des zu codierenden Zeichens. */

void rle2write_rep(header_byte,repeated_byte,repetition_number)
unsigned char header_byte,repeated_byte;
unsigned int repetition_number;

/* Schreibfunktion fr ein wiederholtes Auftreten des gleichen Zeichens.
Erst ab 4 hintereinander folgenden gleichen Zeichen zahlt es sich aus mit  Escapezeichen und Wiederholungen zu arbeiten.
Diese Unterscheidung geschieht bereits in der ersten Zeile. Falls aber der Fall auftritt das es kein unbenutztes Byte gibt,
welches man exklusiv als Escapezeichen verwenden knnte, wird die hier folgenden Konvention angewendet um trotzdem eine eindeutige
Codierung zu erreichen. (Siehe David Bourgin's COMPRESS.TXT (Datei enthalten in codecs13.zip) > *** Second RLE scheme ***). */

{ if (repetition_number<4)
     if (repeated_byte==header_byte)
        { write_byte(header_byte);
          write_byte(repetition_number-1);
          if (repetition_number==3)
             write_byte(header_byte);
        }
     else { register unsigned int i;

            for (i=1;i<=repetition_number;i++)
                write_byte(repeated_byte);
          }
  else { write_byte(header_byte);
         write_byte(repetition_number-1);
         write_byte(repeated_byte);
       }
}

/* Schreibfunktion fr ein nicht-wiederholtes Auftreten des eingelesen Zeichens.
Auch hier mu wieder jener Fall beachtet werden, in dem es kein freies Byte fr ein exklusives Escape Zeichen gibt. 
(Siehe David Bourgin's COMPRESS.TXT > *** Second RLE scheme ***). */

void rle2write_non_rep(header_byte,non_repeated_byte)

unsigned char header_byte,non_repeated_byte;
{ if (non_repeated_byte==header_byte)
     { write_byte(header_byte);
       write_byte(0);
     }
  else write_byte(non_repeated_byte);
}


/* Um den Codiervorgang zu verstehen ist ein Grundlagenwissen der RLE Codierung erforderlich. 
Siehe David Bourgin's COMPRESS.TXT > *** Second RLE scheme ***
Die Variablen der folgenden Funktion sind: "byte1" = 1. eingelesenes Byte. "byte2" = das darauf folgende eingelesene Byte. 
"header_byte" = Escape Zeichen. "frame_size" = Anzahl der Wiederholungen. "lookup_table" = Speicherarray um herauszufinden, welches Byte
im einzulesenden Stream am wenigsten oft vorkommt > Escapezeichenbestimmung. */

void rle2encoding()

{ unsigned char byte1,byte2,header_byte;
  unsigned int frame_size;
  register unsigned int i;
  unsigned long int lookup_table[256];

  if (!end_of_data())        /* Gibt es berhaupt ein Byte zum Einlesen? */
     {                       /* Auf 0 initialisieren des lookup_table */
       (void)memset((char *)lookup_table,0,sizeof(lookup_table));
                             /* This is the same to fill 'lookup_table' to 0.
                                It's fastest than to loop 256 times */
       
/* Routine zum Auswerten welches Byte sich am besten als Escape Zeichen einsetzen lt.
Nachdem ein Escape Zeichen bestimmt wurde, wird es mit "write_byte" auch gleich als 1. Zeichen in die auszugebende codierte Datei geschrieben,
damit der Decodier Routine klar ist, welches das Escape Zeichen ist. */ 

	 while (!end_of_data())
             { byte1=read_byte();
               lookup_table[byte1]++;
             }
       header_byte=0;
       for (i=1;i<=255;i++)
           if (lookup_table[i]<lookup_table[header_byte])
              header_byte=i;
       
 	 write_byte(header_byte);
      
/* Konkreter Beginn des Codierungsverfahren mit dem Einlesen des 1. Bytes und Setzen der Wiederholgen auf 1. */

	 beginning_of_data();
       byte1=read_byte();
       frame_size=1;

/* Falls es nicht das einzige Symbol zum Einlesen war, wird nun solange eingelesen bis das EOF erreicht ist.
D.h. es wird nun das 2. Byte eingelesen. */

       if (!end_of_data())
          { byte2=read_byte();
            frame_size=2;
            do {            

/* Falls 1. und 2. eingelesenes Byte identisch waren wird auf max 255 weitere Wiederholungen untersucht und die Anzahl in der "frame_size"
mitgezhlt. (Mehr knnen auf einmal nicht gespeichert werden. Falls wirklich mehr als 255 Wiederholungen auftreten wird ein neuer Eintrag 
im codierten File erstellt. */

                 if (byte1==byte2)
                    { while ((!end_of_data())&&(byte1==byte2)&&(frame_size<256))
                            { byte2=read_byte();
                              frame_size++;
                            }
/* Bedingung fr den Fall das wirklich mehr als 255 gleiche Zeichen hintereinander vorkommen. */

                      if (byte1==byte2)
                         { rle2write_rep(header_byte,byte1,frame_size);
                           if (!end_of_data())
                              { byte1=read_byte();
                                frame_size=1;
                              }
                           else frame_size=0;
                         }

/* Bedingung fr den Fall das nach 255 gleichen Zeichen Schlu war. */

                      else { rle2write_rep(header_byte,byte1,frame_size-1);
                             byte1=byte2;
                             frame_size=1;
                           }
                    }

/* Fall es an Anfang schon mal berhaupt keine zwei identen Zeichen gegeben hat wird hier auch nur mal das 1. Zeichen in die Ausgabedatei
geschrieben. */

                 else {      
                        rle2write_non_rep(header_byte,byte1);
                        byte1=byte2;
                        frame_size=1;
                      }

/* Mit dieser Funktion wird nach jedem Durchgang berprft ob es noch mindestens 2 weitere Zeichen zum Einlesen gibt oder nicht, 
in dem die "frame_size auf 2 gesetzt wird. Siehe WHILE Bedingung unten. */

                 if (!end_of_data())
                    { byte2=read_byte();
                      frame_size=2;
                    }
               }

/* Weiter einlesen? */

            while ((!end_of_data())||(frame_size>=2));
          }

/* Wurde vorher irgendwo die "frame_size" auf 1 gesetzt ist das ein Anzeichen dafr das nun das letzte Zeichen im Stream erreicht ist, welches
nun hier in die Ausgabedatei geschrieben wird. */

       if (frame_size==1)
          rle2write_non_rep(header_byte,byte1);
     }
}


/* Hauptfunktion dieser RLE-Codierungsroutine. Codiert, erstellt das RLE-codierte (tmp-)file und gibt den Pfad und Namen, des tmp Files zurck.
Die Variablen der folgenden Funktion sind: "*cod_tmp" = Pfad und Name der tmp Datei. */ 

char *rle_cod(char *argv)
{
char *cod_tmp;

/* Erzeugt einen "unique" Dateinamen im angegebenen Verzeichnis. */

if( (cod_tmp = _tempnam( "c:\\", "nice_tmp" ) ) != NULL )
	// printf( "%s is safe to use as a temporary file.\n", cod_tmp );
	;
else
	// printf( "Cannot create a unique filename\n" );
	;

/* Es ist wichtige die Datei im BINARY mode zu ffnen, damit die Steuerzeichen nicht interpretiert werden!!! */

if ( (source_file=fopen(argv,"rb"))==NULL )
{ 
  exit(BAD_FILE_NAME);
}
else if ( (dest_file=fopen(cod_tmp,"wb"))==NULL )
{ 
	// printf ("konnte tmp datei nicht erstellen");
	exit(BAD_FILE_NAME);
}
else 

/* Codierungsteil starten */

rle2encoding();

fclose(source_file);
fclose(dest_file);

// printf("Execution of codrle2 completed.\n");

/* bergabe des tmp Dateinames */

return (cod_tmp);
}