/*
File: rle_dcod.c
Zweck: RLE DeCoding Routine fr das niCE Projekt.
Funktion: bernimmt als Argument den tmp Dateinamen und den gewnschten Ausgabenamen sowie Generierung der dekomprimierten Datei.
Author: Christian Sam / christian@niceshoes.cc
Datum: 11.05.2003

Hinweis: Teile dieser RLE Funktion beruhen auf der RLE Routine2 von David Bourgin / david.bourgin@ufrima.imag.fr
*/


/* Einbinden eines Standardheaders, bentigt werden mindestens: <stdio.h>, <memory.h>, <stdlib.h> */
#include "../nice.h"


/* Definition von Standard Error Codes */
#define NO_ERROR      0
#define BAD_FILE_NAME 1
#define BAD_ARGUMENT  2

/* Definition von Konstanten */
#define FALSE 0
#define TRUE  1


/* Definition von Globalen Variablen und Funktionen, welche fr das Handling und Einlesen mit geffneten Dateien bentigt werden 

!!! Wichtig, diesen Teil unbedingt auf Verstndnis durchlesen !!!

Es werden "end_of_data" und "read_byte" gemeinsam benutzt um Bytes einzulesen. Bytes werden in einer While Funktion eingelesen die solang 
ausgefhrt wird "while (!end_of_data())". Die Bytes werden in "val_byte_stored" gespeichert, als Flag das das Byte eingelesen wurde
wird "byte_stored_status" auf TRUE gesetzt. Im nchsten Durchgang wird in der Funktion "read_byte" bei gesetzem "byte_stored_status" Flag,
das eingelesen Byte von "val_byte_stored" auf unsigned char umgecastet und an die aufrufende Funktion bergeben, es wird auerdem das Flag
von "byte_stored_status" mittels FALSE wieder zurckgesetzt, soda im nchsten Durchgang der While Schleife wieder ein Byte eingelesen wird.
Die Bytes werden also solange an die aufrufende Funktion zurckgegeben, solang das Flag in der Funktion "end_of_date" gesetzt ist.

Die Funktion "write_byte" wird benutzt um den nun RLE komprimierten Stream in ein Ausgangsfile zurckzuschreiben. */

FILE *source_file,*dest_file;

/* byte_stored_status sollte am Anfang auf FALSE gesetzt werden, dies passiert aber bereits im rle_cod.c */

int byte_stored_status;
int val_byte_stored;

#define end_of_data()  (byte_stored_status?FALSE:!(byte_stored_status=((val_byte_stored=fgetc(source_file))!=EOF)))
#define read_byte()  (byte_stored_status?byte_stored_status=FALSE,(unsigned char)val_byte_stored:(unsigned char)fgetc(source_file))
#define write_byte(byte)  ((void)fputc((byte),dest_file))

/*Die Variablen der folgenden Funktion sind: "header_byte" = Escape Zeichen. */

/* Returned parameters: None
   Action: Decompresses with RLE type 2 method all bytes read by the function read_byte
   Errors: An input/output error could disturb the running of the program
*/
void rle2decoding()
{
  unsigned char header_byte,byte_read,byte_to_repeat;
  register unsigned int i;

  if (!end_of_data())

/* Erstes Byte das eingelesen wird, ist laut Konvention das Escape Zeichen. */

   { header_byte=read_byte();
       do {                  

/* Umwandeln der RLE-Codierung laut Konvention (Siehe David Bourgin's COMPRESS.TXT 
(Datei enthalten in codecs13.zip) > *** Second RLE scheme ***). */

            if ((byte_read=read_byte())==header_byte)
               { byte_read=read_byte();
                 if (byte_read<3)
                    for (i=0;i<=byte_read;i++)
                        write_byte(header_byte);
                 else { byte_to_repeat=read_byte();
                        for (i=0;i<=byte_read;i++)
                            write_byte(byte_to_repeat);
                      }
               }
            else write_byte(byte_read);
          }
       while (!end_of_data());
     }
}

/* Hauptfunktion dieser RLE-Decodierungsroutine. bernimmt als Argument den tmp Dateinamen und den gewnschten Ausgabenamen
sowie Generierung der dekomprimierten Datei. */

void rle_dcod(char *dcod_tmp, char *argv)
{
 if ((source_file=fopen(dcod_tmp,"rb"))==NULL)
 { 
   exit(BAD_FILE_NAME);
 }
 else 
 ; 
 if ((dest_file=fopen(argv,"wb"))==NULL)
 { 
   exit(BAD_FILE_NAME);
 }
 else 
 rle2decoding();

 fclose(source_file);
 fclose(dest_file);
 
 //printf("Execution of dcodrle2 completed.\n");
}