//*****************************************************//
//Dateiname: save_tree.c
//Autor: Walter Scherer
//Projekt: niCE - nice insignificant compression engine
//Zweck: Speichert den Huffman-Baum, wobei die Traversierungsfunktion von TraverseTree verwendet wird
//Datum der letzten Aenderung: 21.05.2003
//*****************************************************//

//spezifische INCLUDES
#include "../nice.h"
#include "../huffman.h"
#include "../globals.h"

//***************************************************************************
//
//					Funktionen
//
//***************************************************************************

/**
 *
 *	@function: bersetzt den Baum in speicherbare Bitsequences
 *	@params:
 *				pt_file... Zeiger auf die zum Schreiben geffnete Datei
 *				pt_RootNode ... Wurzelknoten des Baums
 *				BufferBitSequence ... Durchgereichte BitSequence fr TraverseTreeforSave(...)
 *	@return value:
 *				BufferBitSequence ... gibt Coderest weiter
 */
BitSequence SaveTree(FILE *pt_file, Node *pt_RootNode, BitSequence BufferBitSequence)
{
	TraverseTreeforSave(pt_RootNode, "", &SaveNodetoFile, &BufferBitSequence, pt_file);						//Traversiert den Baum und speichert nach der Reihe die Bitsequenzen ab
	return BufferBitSequence;		//Den am Schluss berbleibenden Coderest an CompressFile(...) zurckgeben (TraverseTreeforSave(...) schreibt den Coderest (Lnge < 8) nicht mehr in die Datei)
}

/**
 *
 *	@function: ffnet die Datei und bereitet sie auf den Schreibzugriff vor
 *	@params:
 *				file_name_source ... Dateiname der zu lesenden Datei (kommt von der Parameterbehandlung)
 *				file_name_target ... Dateiname der Datei, in die geschrieben wird (beide Dateien werden in dieser Funktion geffnet)
 *				pt_RootNode ... WurzelKnoten des Baums
 *				pt_array_codetable ... Weitergabe der Codiertabelle an CompressFile (kommt aus main)
 *	@return value:
 *				KEINER
 *
 */
void OpenFiletoSave(char *file_name_source, char *file_name_target, Node *pt_RootNode, BitSequence *pt_array_codetable, char Continue)
{
	FILE *pt_file_source;						//File-Pointer fr Quelldatei
	FILE *pt_file_target;						//File-Pointer fr Zieldatei
	BitSequence BufferBitSequence;				//BitSequence fr den in die Datei zu schreibenden Code
	int temp_code=0;							//Initialisierung des Codes
	BufferBitSequence.length = 0;				//Lnge der BufferBitSequence initialisieren
	BufferBitSequence.Code = &temp_code;		//Initialisierung des Codes

	if (((pt_file_source=(fopen(file_name_source, "rb")))==NULL) || ((pt_file_target=(fopen(file_name_target, "wb")))==NULL))	//falls eine der beiden Dateien nicht geffnet werden konnten...
	{
		printf("huffman:save_tree.c: could not open files.\n");
		temp_num_error++;
		return;
	}
	BufferBitSequence=SaveTree(pt_file_target, pt_RootNode, BufferBitSequence);		//Baum abspeichern
	CompressFile(pt_file_source, pt_file_target, pt_array_codetable, BufferBitSequence, Continue);		//Datei komprimieren
	fclose(pt_file_source);				//Dateien schlieen
	fclose(pt_file_target);
}
