//*****************************************************//
//Dateiname: BitOperations.c
//Autor: Ptschner Stephan
//Projekt: niCE - nice insignificant compression engine
//Zweck: Funktionen, die Bitmanipulationen vorzunehmen und einzelne Bits aus Daten auslesen knnen
//Datum der letzten Aenderung: 06.05.2003
//*****************************************************//

//Eigene spezifische INCLUDES
#include "../nice.h"
#include "../huffman.h"

//***************************************************************************
//
//					Funktionen
//
//***************************************************************************

/**
 *
 *	@function: Erzeugt aus einer Zeichenkette eine BitSequenz, dh. wandelt eine Zeichenkette bestehend aus "0" u. "1" in eine Zahl um
 *	@params:
 *				String ... Zeichenkette, welche aus Nullen und Einsen bestehen muss
 *				pt_GeneratedCode ... Zeiger auf die Bitsequenz in welcher die umgewandelte Zeichenfolge als Zahl abgespeichert wird
 *	@return value:
 *				-(Zahl) bei einem Fehler
 *				0 bei normalem Abschluss
 *
 */
int CreateBitSequence (char *String, BitSequence *pt_GeneratedCode)
{
	unsigned int run_a;
	unsigned int temp_StringLength = strlen(String);	// Variable zum speichern der Bitlnge der zu erstellenden Zahl
	unsigned int *temp_Buffer;				// temp. Variable in welcher die neue (Binr-)Zahl erstellt wird
	char temp_SizeOfArray = MEMSIZE(temp_StringLength);	// Makro berrechnet wieviele int-Datentypen bentigt werden um die Bitfolge "String" abzubilden

	if ( temp_StringLength > 255)	// Wenn versucht wird eine Bitfolge lnger als 255 abzubilden -> Fehlerrckgabe (Bitsequence->length: unsigned char -> kann hchsten 256 annehmen)
		return -2;

	pt_GeneratedCode->length = temp_StringLength;		// Setzen der Lnge der Bitfolge in der Bitsequenz
	pt_GeneratedCode->Code = (int *) malloc(temp_SizeOfArray);	// Reservieren von "temp_SizeOfArray"-int-Datenblcken zum Speichern der erzeugten Zahl
	memset(pt_GeneratedCode->Code, 0, temp_SizeOfArray * sizeof(int));	// initialisieren der Zahl mit lauter Nullen

	// temp_Buffer durchluft das dynamische Feld auf das pt_GeneratedCode->Code zeigt von hinten (temp_SizeOfArray - 1)
	temp_Buffer = pt_GeneratedCode->Code + (temp_SizeOfArray - 1);

	// Jedes einzelne Zeichen der Zeichenfolge String wird berprft und in die DezZahl "eingearbeitet"
	for (run_a =  0; run_a < temp_StringLength; run_a++)	// Durchlaufen der Zeichenkette (bestehend aus "0" und "1")
	{
		switch(String[run_a])
		{
			// Aktueller Buchstabe ist eine '0'
			case '0':
				// shiften der Zahl nach links (letztes Bit wird zum "berschreiben" frei)
				*temp_Buffer <<= 1;
				// AND-Verknpfung mit einer Zahl hnlich (111111110 = (not) 000000001 = (not) 1)
				// ersten Bit werden nicht verndert, letztes Bit wird auf 0 gesetzt (a & 0 = 0 und a & 1 = a)
				*temp_Buffer &= (~ (int) 1);
				break;
			case '1':
				*temp_Buffer <<= 1;
				// OR-Verknpfung mit "1" -> ersten Bit werden nicht verndert (a | 0 = a); letztes Bit wird auf 1 gesetzt (a | 1 = 1)
				*temp_Buffer |= 1;
				break;
			default: // Abbruch der Funktion falls sich in der Zeichenkette ein Buchstabe ungleich '0' bzw '1' befindet
				return -1;
		}
		// Wenn ein int vollgeschrieben wurde ...
		if(run_a % (sizeof(int) * 8) == 0  && run_a > 0)
			temp_Buffer--;	// ... zeigt temp_Buffer auf den nchsten int
	}

	return 0;
}

/**
 *
 *	@function: Potenziert eine Zahl hoch einem Exponent (entspricht der Standardfunktion pow())
 *	@params:
 *				base ^ exponent
 *				base ... Basis der Potenzrechnung
 *				exponent ... Exponent der Potenzrechnung
 *	@return value:
 *				Ergebnis der Rechnung
 *
 */
double PowerOf (long base, int exponent)
{
	// Sonderfall Exponent == 0 -> Ergebnis muss 1 sein
	if(exponent == 0)
		return 1;

	// Sonderfall Exponent grsser als 0 -> Schleife um zu potenzieren
	if(exponent > 0)
	{
		double temp_value = base;
		int run_a;
		
		// potenziert Zwischenergebnis mit der Basis
		for(run_a = 1; run_a < exponent;run_a ++)
			temp_value *= base;

		return temp_value;
	}


	// Sonderfall Exponent kleiner als 0 -> Schleife um zu potenzieren (basis -> 1/basis)
	if(exponent < 0)
	{
		double temp_value = 1 / (double)base;
		double temp_base = temp_value;
		int run_a;

		for(run_a = 1; run_a < (-exponent);run_a ++)
			temp_value *= (temp_base);

		return temp_value;
	}

	return 0;
}

/**
 *
 *	@function: Kontrolliert ob ein bestimmtes Bit einer Integer Variablen gesetzt ist
 *	@params:
 *				CheckSequence ... Integer von welchem geprft wird ob das n-te Bit gesetzt ist
 *				CheckBitNr ... Nummer des Bits welches geprft werden soll
 *	@return value:
 *				gibt 1 zurck falls das Bit "CheckBitNr" gesetzt ist, ansonsten 0
 *
 */
int IsBitSet (int CheckSequence, char CheckBitNr)
{
	// 2^x ergibt eine Binrzahl mit gesetztem x'ten Bit -> Und Verknpfung ergibt 0 falls diese Bit nicht gesetzt ist
	return ((CheckSequence & (int) PowerOf(2, CheckBitNr - 1)) ? 1 : 0);
}

/**
 *
 *	@function: Kontrolliert ob ein bestimmtes Bit einer BitSequenz gesetzt ist
 *	@params:
 *				CheckSequence ... BitSequenze von welcher geprft wird ob das n-te Bit gesetzt ist
 *				CheckBitNr ... Nummer des Bits welches geprft werden soll
 *	@return value:
 *				gibt 1 zurck falls das Bit "CheckBitNr" gesetzt ist, ansonsten 0
 *
 */
int IsBitSetOfBS (BitSequence CheckSequence, unsigned char CheckBitNr)
{
	if(CheckBitNr > CheckSequence.length || CheckBitNr == 0)
		return 0;
	
	// Bit aus BitSequence wird berprft
	{	
		// temp_Bitsequenz wird auf den Integer gesetzt in welchem das zu berprfende Bit ist (BitSequence.Code zeigt auf ein Feld aus Integern!!!)
		BitSequence *BufferSequence = &CheckSequence + MEMSIZE(CheckBitNr) - 1;
		// Wenn Checkbitnr % 32 == 0 dann soll 32 und nicht 0 ausgegeben werden
		CheckBitNr = (CheckBitNr % (sizeof(int) * 8) == 0 ? sizeof(int) * 8 : CheckBitNr % (sizeof(int) * 8));
		// Verwenden der vorhergehenden Funktion und gleichzeitge Rckgabe als Return-Wert
		return (IsBitSet(*(BufferSequence->Code), (char)CheckBitNr));
	}
	

}


/**
 *
 *	@function: Shifted eine ganze BitSequenz um 1 Bit nach links
 *	@params:
 *				*ActualBitSequence ... BitSequenz die geshifted werden soll
 *	@return value:
 *
 */
void ShiftLeft(BitSequence *ActualBitSequence)
{
	// Wenn ActualBitSequenz auf ein Feld von Integerwerten zeigt
	if(ActualBitSequence->length > sizeof(int) * 8 - 1) // minus 1, da 1 Bit ja "geshifted" wird
	{
		BitSequence *BufferSequence;
		
		// Durchlaufen des Integer-Felds ...
		for(BufferSequence = ActualBitSequence; BufferSequence == ActualBitSequence + (MEMSIZE(ActualBitSequence->length) - 1); BufferSequence++)
		{
			// ... und Shiften jedes einzelnen Int's nach links
			*(BufferSequence->Code) <<= 1;
			// bernehmen eines herausfallenden Bits in den nchsten Integer
			*(BufferSequence->Code) &= IsBitSet(*((BufferSequence + 1)->Code), 8 * sizeof(int));
		}
	}

	*(ActualBitSequence->Code) <<= 1;
	ActualBitSequence->length++;	// Erhhen der Lnge um 1 da die Bitfolge um eins nach links geschoben wurde

}