//*****************************************************//
//Dateiname: FileOperations.c
//Autor: Ptschner Stephan
//Projekt: niCE - nice insignificant compression engine
//Zweck: Funktionen zum Bitweisen Schreiben und Lesen in/aus eine(r) Datei
//Datum der letzten Aenderung: 11.05.2003
//*****************************************************//

//Eigene spezifische INCLUDES
#include "../nice.h"
#include "../huffman.h"


//***************************************************************************
//
//					Funktionen
//
//***************************************************************************

/**
 *
 *	@function: Versucht eine neue Bitsequenz an eine Buffersequenz zu hngen; wird diese dabei "voll", so wird sie in die Datei geschrieben und kann somit wieder befllt werden
 *	@params:
 *				pt_File ... Zeiger auf Datei in der die Schreiboperationen durchgefhrt werden sollen
 *				pt_BufferBitSequence ... Buffersequenz, die Programmintern solange nicht in die Datei geschrieben wird bis sie die volle Lnge (1 int) erreicht hat
 *				pt_ActualBitSequence ... BitSequenz, die an BufferSequenz angefgt werden soll und in die Datei geschrieben werden soll
 *	@return value:
 *				Fehlerrckgabe: "0" bei keinem Fehler; "1" bei Fehler
 *
 */
int WriteBitSequenceToFile(FILE *pt_file, BitSequence *pt_BufferBitSequence, BitSequence *pt_ActualBitSequence)
{
	unsigned char run_a;
	char temp_ReturnValue = 0;
	// Wenn die Buffersequenz bereits voll ist soll Sie geleert werden
	if(pt_BufferBitSequence->length == sizeof(int) * 8)
	{
		// Schreiben des vollen Buffers in die Datei (Grsse: 1 int)
		temp_ReturnValue = fwrite(pt_BufferBitSequence->Code, sizeof(int), 1, pt_file);

		// Buffersequenz leeren (neu initialisieren)
		*(pt_BufferBitSequence->Code) = 0;
		pt_BufferBitSequence->length = 0;
	}

	// Falls keine Bitsequenz angegeben wurde, die angehngt werden soll, Abbruch der weiteren Funktionsausfhrung
	if(pt_ActualBitSequence == NULL)
		return temp_ReturnValue;

	// Durchlaufen des gesammten Inhalts der aktuellen (zu schreibenden) Bitsequenz um jedes Bit einzeln der Bitsequenz hinzuzufgen 
	// und zu berprfen ob die BufferBitSequenz schon voll ist
	for(run_a = pt_ActualBitSequence->length; run_a > 0; run_a--)
	{
		// Im Buffer wird der letzte Platz frei gemacht (- um eine Position nach links geshiftet)
		ShiftLeft(pt_BufferBitSequence);
		// Hinzufgen der <run_a>'ten Bits aus der Aktuellen - Bitsequenz auf die letzte Stelle der BufferBitSequenz
		*(pt_BufferBitSequence->Code) |= IsBitSetOfBS(*pt_ActualBitSequence, run_a);

		// Falls die Buffersequenz durch Hinzufgen des Bits gefllt wurde, soll sie geleert werden
		if(pt_BufferBitSequence->length == sizeof(int) * 8)
		{
			temp_ReturnValue = fwrite(pt_BufferBitSequence->Code, sizeof(int), 1, pt_file);
			*(pt_BufferBitSequence->Code) = 0;
			pt_BufferBitSequence->length = 0;
		}
	}
	return temp_ReturnValue;
}

/**
 *
 *	@function: Liest ein Bit aus einer Datei; dabei wird in Wirklichkeit ein Byte gelesen und bei jedem Aufruf der Funktion ein Bit dieses Bytes zurckgegeben
 *	@params:
 *				pt_File ... Zeiger auf Datei in der die Leseoperationen durchgefhrt werden sollen
 *				pt_BufferBitSequence ... Buffersequenz, die funktionsintern dazu benutzt wird, dass nur 1 Bit zurckgegeben werden kann und die restlichen Bits nicht verloren gehen
 *	@return value:
 *				'1' ... das gelesene Bit ist eine 1
 *				'0' ... das gelesene Bit ist eine 0
 *
 */
char GetBitFromFile(FILE *pt_File, BitSequence *pt_BufferSequence)
{
	// Wenn die BufferSequenz bereits leer ist, ...
	if (pt_BufferSequence->length == 0)
	{
		// ... soll wieder ein Byte aus der Datei ausgelesen werden ...
		int temp_returnValue;
		if((temp_returnValue = GetBitsequenceFromFile(pt_File, pt_BufferSequence)) < 0)
			return temp_returnValue;
	}

	return IsBitSetOfBS (*pt_BufferSequence, pt_BufferSequence->length--);
}

/**
 *
 *	@function: Ident mit der Funktion "GetBitFromFile", jedoch wird der Lngencounter der Bitsequenz weder manipuliert noch beachtet
 *	@params:
 *				pt_File ... Zeiger auf Datei in der die Leseoperationen durchgefhrt werden sollen
 *				pt_BufferBitSequence ... Buffersequenz, die funktionsintern dazu benutzt wird, dass nur 1 Bit zurckgegeben werden kann und die restlichen Bits nicht verloren gehen
 *	@return value:
 *				'1' ... das gelesene Bit ist eine 1
 *				'0' ... das gelesene Bit ist eine 0
 *
 */
int GetBitsequenceFromFile(FILE *pt_File, BitSequence *pt_BufferSequence)
{
	if(fread(pt_BufferSequence->Code, sizeof(int), 1, pt_File) != 1)
	{
		if (feof(pt_File))
			return -2;
		else
			return -1;
	}

	// und die Lnge auf 8 gesetzt werden ()
	pt_BufferSequence->length = sizeof(int) * 8;
	return 0;
}

/**
 *
 *	@function: Liest ein Byte aus einer Datei unter Verwendung des Buffers
 *	@params:
 *				pt_File ... Zeiger auf Datei in der die Leseoperationen durchgefhrt werden sollen
 *				pt_BufferBitSequence ... Buffersequenz, die funktionsintern dazu benutzt wird, dass nur 1 Bit zurckgegeben werden kann und die restlichen Bits nicht verloren gehen
 *	@return value:
 *				ausgelesenes Byte aus der Datei
 *
 */
char GetByteFromFile(FILE *pt_File, BitSequence *pt_BufferSequence)
{
	char temp_OneBit;
	char temp_ReturnValue = 0;

	int run_a;
	for (run_a = 0; run_a < 8; run_a++)
	{
		// GetBitfromFile wird verwendet, da sonst beim Auslesen eines Bytes der Puffer nicht bercksichtigt werden wrde
		temp_OneBit = GetBitFromFile(pt_File, pt_BufferSequence);
		if(temp_OneBit == -2)
			return temp_ReturnValue;
		else
		{
			temp_ReturnValue <<= 1;
			temp_ReturnValue |= temp_OneBit;
		}
	}
	return temp_ReturnValue;
}
