//*****************************************************//
//Dateiname: TraverseTree.c
//Autor: Ptschner Stephan, Walter Scherer
//Projekt: niCE - nice insignificant compression engine
//Zweck: Datei mit Funktion, welche einen Baum traversiert und bei jedem Knoten eine bestimmte Funktion aufruft, und dazugehrige Funktionen
//Datum der letzten Aenderung: 21.05.2003
//*****************************************************//

//Eigene spezifische INCLUDES
#include "../nice.h"
#include "../huffman.h"

//***************************************************************************
//
//					Funktionen
//
//***************************************************************************

/**
 *
 *	@function: Traversiert eine Baumstruktur rekursiv (postorder (linkes Blatt, rechtes Blatt, Wurzel))
 *	@params:
 *				pt_RootNode ... Zeiger auf den Wurzel-Knoten der Baumstruktur
 *				pt_Depth ... Indikator fr die Tiefe des Wurzelknotens in der Baumstruktur -> bei absolutem WurzelElement mit "" zu belegen
 *				(*ManipulateNode) ... Zeiger auf eine Funktion, die jedes Wurzelelement bearbeiten kann/soll
 *						pt_ActualNode ... Zeiger auf den aktuell zu bearbeitenden Knoten
 *						pt_Depth ... Tiefe des aktuellen Knotens in der Baumstruktur ("0" entspricht "links abbiegen" in der Baumstruktur und "1" rechts)
 *				pt_ExtraData ... Zusatzinformationen, welche an die aufgerufene Funktion weiterbergeben werden, um dieser zu ermglichen mit der aufrufenden Funktion Daten auszutauschen
 *	@return value:
 *				KEINER
 *
 */
void TraverseTree(Node *pt_RootNode, char *pt_Depth, void (*ManipulateNode)(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData), void *pt_ExtraData)
{
	int run_a;
	char temp_Depth[256];	// Variable zum lokalen Ablegen der Tiefe des aktuellen Knotens

	strcpy(temp_Depth, pt_Depth);	// Abspeichern der Baumtiefe in der eben erzeugten lokalen Variable
	
	for (run_a = 0; run_a < 2; run_a++)	// Schleife, die rekursiv in die Kinderknoten "absteigt", falls diese vorhanden sind
	{
		char temp_Buffer[2];	// Speichervariable den ASCII Wert des Kindknotens ("0" / "1")
		char temp_Buffer2[256];		// Speichervariable die die aktuelle Baumtiefe mit der des Kindknotens verknpft (wird bentigt, da strcat() auch die bergebene Variable verndert)
		strcpy(temp_Buffer2, temp_Depth);	// Kopieren der aktuellen Baumtiefe in den Buffer


		(void) itoa(run_a, temp_Buffer, 10);	// Speichern von "0" / "1" abhngig vom Kindknoten der "aufgerufen" wird

		if (pt_RootNode->pt_Down[run_a] != NULL)		// Falls es einen Kindknoten gibt ...
			TraverseTree(pt_RootNode->pt_Down[run_a], strcat(temp_Buffer2, temp_Buffer), (*ManipulateNode), pt_ExtraData); // ... soll rekursiv in diesen "eingesprungen" werden und die Parameter werden weiterbergeben
	}
	(*ManipulateNode) (pt_RootNode, temp_Depth, pt_ExtraData);	// Fr jeden Knoten wird die ber Parameter bergebene Funktion aufgerufen

}

/**
 *
 *	@function: Traversiert eine Baumstruktur rekursiv (preorder (Wurzel, linkes Blatt, rechtes Blatt))
 *	@params:
 *				pt_RootNode ... Zeiger auf den Wurzel-Knoten der Baumstruktur
 *				pt_Depth ... Indikator fr die Tiefe des Wurzelknotens in der Baumstruktur -> bei absolutem WurzelElement mit "" zu belegen
 *				(*ManipulateNode) ... Zeiger auf eine Funktion, die jedes Wurzelelement bearbeiten kann/soll
 *						pt_ActualNode ... Zeiger auf den aktuell zu bearbeitenden Knoten
 *						pt_Depth ... Tiefe des aktuellen Knotens in der Baumstruktur ("0" entspricht "links abbiegen" in der Baumstruktur und "1" rechts)
 *				pt_ExtraData ... Zusatzinformationen, welche an die aufgerufene Funktion weiterbergeben werden, um dieser zu ermglichen mit der aufrufenden Funktion Daten auszutauschen
 *	@return value:
 *				KEINER
 *
 */
void TraverseTreePreorder(Node *pt_RootNode, char *pt_Depth, void (*ManipulateNode)(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData), void *pt_ExtraData)
{
	int run_a;
	char temp_Depth[256];	// Variable zum lokalen Ablegen der Tiefe des aktuellen Knotens

	strcpy(temp_Depth, pt_Depth);	// Abspeichern der Baumtiefe in der eben erzeugten lokalen Variable

	(*ManipulateNode) (pt_RootNode, temp_Depth, pt_ExtraData);	// Fr jeden Knoten wird die ber Parameter bergebene Funktion aufgerufen

	
	for (run_a = 0; run_a < 2; run_a++)	// Schleife, die rekursiv in die Kinderknoten "absteigt", falls diese vorhanden sind
	{
		char temp_Buffer[2];	// Speichervariable den ASCII Wert des Kindknotens ("0" / "1")
		char temp_Buffer2[256];		// Speichervariable die die aktuelle Baumtiefe mit der des Kindknotens verknpft (wird bentigt, da strcat() auch die bergebene Variable verndert)
		strcpy(temp_Buffer2, temp_Depth);	// Kopieren der aktuellen Baumtiefe in den Buffer


		(void) itoa(run_a, temp_Buffer, 10);	// Speichern von "0" / "1" abhngig vom Kindknoten der "aufgerufen" wird

		if (pt_RootNode->pt_Down[run_a] != NULL)		// Falls es einen Kindknoten gibt ...
			TraverseTree(pt_RootNode->pt_Down[run_a], strcat(temp_Buffer2, temp_Buffer), (*ManipulateNode), pt_ExtraData); // ... soll rekursiv in diesen "eingesprungen" werden und die Parameter werden weiterbergeben
	}

}

/**
 *
 *	@function: Traversiert eine Baumstruktur rekursiv (preorder (linkes Blatt, rechtes Blatt, Wurzel))
 *	@params:
 *				pt_RootNode ... Zeiger auf den Wurzel-Knoten der Baumstruktur
 *				pt_Depth ... Indikator fr die Tiefe des Wurzelknotens in der Baumstruktur -> bei absolutem WurzelElement mit "" zu belegen
 *				(*ManipulateNode) ... Zeiger auf eine Funktion, die jedes Wurzelelement bearbeiten kann/soll
 *						pt_ActualNode ... Zeiger auf den aktuell zu bearbeitenden Knoten
 *						pt_Depth ... Tiefe des aktuellen Knotens in der Baumstruktur ("0" entspricht "links abbiegen" in der Baumstruktur und "1" rechts)
 *						pt_ExtraData ... Zusatzinformationen, welche an die aufgerufene Funktion weiterbergeben werden, um dieser zu ermglichen mit der aufrufenden Funktion Daten auszutauschen
 *						pt_file ... Speicherdatei
 *				pt_file ... Speicherdatei
 *	@return value:
 *				KEINER
 *
 */
void TraverseTreeforSave(Node *pt_RootNode, char *pt_Depth, void (*ManipulateNode)(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file), void *pt_ExtraData, FILE *pt_file)
{
	int run_a;
	char temp_Depth[256];	// Variable zum lokalen Ablegen der Tiefe des aktuellen Knotens

	strcpy(temp_Depth, pt_Depth);	// Abspeichern der Baumtiefe in der eben erzeugten lokalen Variable

	(*ManipulateNode) (pt_RootNode, temp_Depth, pt_ExtraData, pt_file);	// Fr jeden Knoten wird die ber Parameter bergebene Funktion aufgerufen
	
	for (run_a = 0; run_a < 2; run_a++)	// Schleife, die rekursiv in die Kinderknoten "absteigt", falls diese vorhanden sind
	{
		char temp_Buffer[2];	// Speichervariable den ASCII Wert des Kindknotens ("0" / "1")
		char temp_Buffer2[256];		// Speichervariable die die aktuelle Baumtiefe mit der des Kindknotens verknpft (wird bentigt, da strcat() auch die bergebene Variable verndert)
		strcpy(temp_Buffer2, temp_Depth);	// Kopieren der aktuellen Baumtiefe in den Buffer


		(void) itoa(run_a, temp_Buffer, 10);	// Speichern von "0" / "1" abhngig vom Kindknoten der "aufgerufen" wird

		if (pt_RootNode->pt_Down[run_a] != NULL)		// Falls es einen Kindknoten gibt ...
			TraverseTreeforSave(pt_RootNode->pt_Down[run_a], strcat(temp_Buffer2, temp_Buffer), (*ManipulateNode), pt_ExtraData, pt_file); // ... soll rekursiv in diesen "eingesprungen" werden und die Parameter werden weiterbergeben
	}
}

/**
 *
 *	@function: Funktion, die von TraverseTree aufgerufen werden kann, um ein HuffmanCodetabelle zu erzeugen
 *	@params:
 *				pt_ActualNode ... Zeiger auf den zu bearbeitenden Knoten
 *				pt_Depth ... Indikator fr die Tiefe des Knotens in der Baumstruktur ("0" entspricht "links abbiegen" in der Baumstruktur und "1" rechts)
 *	@return value:
 *				KEINER
 *
 */
void AddNodeToCodingTable(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData)
{
	// Kontrolle, ob es sich um einen Blattknoten handelt (SimpleNode = 1) oder einen Knoten in der Mitte des Baumes
	// fr die CodierTabelle sind nur Blattknoten von Interesse
	if (pt_ActualNode->iSimpleNode == LEAFNODE)
	{
		BitSequence temp_GeneratedCode;		//Zwischenspeicher fr die aus dem String ptDepth gewonnene Bitsequenz
		if (CreateBitSequence (pt_Depth, &temp_GeneratedCode))		//Erzeugen der Bitsequenz
			printf("Fehler: Code von \'%c\' ist zu lang (%s) \n", pt_ActualNode->cContent, pt_Depth);
		
		// In ExtraData befindet sich der Pointer auf die Kodiertabelle
		// ExtraData ist jedoch vom Typ (void *) -> muss umgecastet werden; Weiters wird das Element Nr.: (pt_ActualNode->cContent) angesprochen und gesetzt
		(*((BitSequence *) pt_ExtraData + pt_ActualNode->cContent)).Code = temp_GeneratedCode.Code;
		(*((BitSequence *) pt_ExtraData + pt_ActualNode->cContent)).length = temp_GeneratedCode.length;
	}
}

/**
 *
 *	@function: Funktion, die von TraverseTree aufgerufen werden kann, um die Codesequenzen zu speichern
 *	@params:
 *				pt_ActualNode ... Zeiger auf den zu bearbeitenden Knoten
 *				pt_Depth ... Indikator fr die Tiefe des Knotens in der Baumstruktur ("0" entspricht "links abbiegen" in der Baumstruktur und "1" rechts)
 *				pt_ExtraData ... Speichert die BufferBitSequence, aber nur hier
 *				pt_file ... Speicherdatei
 *	@return value:
 *				KEINER
 *
 */
void SaveNodetoFile(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file)
{
	int temp_code = 1;									//statische Integer-Variable fr Code (darauf zeigt dann der Pointer aus der Struktur BitSequence)
	BitSequence ActualBitSequence;						//ActualBitSequence enthlt nur jeweils entweder 1 fr einen Blattknoten oder 0 fr einen inneren Knoten
	ActualBitSequence.Code = &temp_code;				//neuerliche Initialisierung des momentanen Codes
	ActualBitSequence.length = 1;
	if (pt_ActualNode->iSimpleNode == LEAFNODE)			//wenn Knoten ein Blattknoten ist, dann muss nach der Information '1' auch noch die Buchstabeninformation hinzugefgt werden
	{
		*(ActualBitSequence.Code) = 1;				//Code der aktuellen Bitsequenz auf '1' setzen
		WriteBitSequenceToFile(pt_file, ((BitSequence *) pt_ExtraData), &ActualBitSequence);		//Integerschreibfunktion aufrufen (schreibt Bits mit Integerlnge in die Datei)
		*(ActualBitSequence.Code) = pt_ActualNode->cContent;
		ActualBitSequence.length = 8;
		WriteBitSequenceToFile(pt_file, ((BitSequence *) pt_ExtraData), &ActualBitSequence);		//Character schreiben (schreibt Character in die Datei)
	}
	else
	{
		*(ActualBitSequence.Code) = 0;				//Code der aktuellen Bitsequenz auf '0' setzen
		WriteBitSequenceToFile(pt_file, ((BitSequence *) pt_ExtraData), &ActualBitSequence);			//Integerschreibfunktion aufrufen (schreibt Bits mit Integerlnge in die Datei)
	}
}

/**
 *
 *	@function: Funktion, die von TraverseTree aufgerufen werden kann, um die Knoten zu lschen
 *	@params:
 *				pt_ActualNode ... Zeiger auf den zu bearbeitenden Knoten
 *				pt_Depth ... (eig. unntig beim lschen der Knoten) Indikator fr die Tiefe des Knotens in der Baumstruktur ("0" entspricht "links abbiegen" in der Baumstruktur und "1" rechts)
 *	@return value:
 *				KEINER
 *
 */
void DeleteNode (Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData)
{
	// Lscht den aktuellen Knoten
	free(pt_ActualNode);
}

/**
 *
 *	@function: Traversiert eine Baumstruktur rekursiv (preorder (linkes Blatt, rechtes Blatt, Wurzel))
 *				wird benutzt um den Baum aus einer komprimierten Datei auszulesen und aufzubauen
 *	@params:
 *				pt_RootNode ... Zeiger auf den Wurzel-Knoten der Baumstruktur
 *				pt_Depth ... Indikator fr die Tiefe des Wurzelknotens in der Baumstruktur -> bei absolutem WurzelElement mit "" zu belegen
 *				(*ManipulateNode) ... Zeiger auf eine Funktion, die jedes Wurzelelement bearbeiten kann/soll
 *						pt_ActualNode ... Zeiger auf den aktuell zu bearbeitenden Knoten
 *						pt_Depth ... Tiefe des aktuellen Knotens in der Baumstruktur ("0" entspricht "links abbiegen" in der Baumstruktur und "1" rechts)
 *						pt_ExtraData ... Zusatzinformationen, welche an die aufgerufene Funktion weiterbergeben werden, um dieser zu ermglichen mit der aufrufenden Funktion Daten auszutauschen
 *						pt_file ... Speicherdatei
 *				pt_file ... Speicherdatei
 *	@return value:
 *				KEINER
 *
 */
void TraverseTreeforRead(Node *pt_RootNode, char *pt_Depth, void (*ManipulateNode0)(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file) , void (*ManipulateNode1)(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file), void *pt_ExtraData, FILE *pt_file)
{
	int run_a;
	char temp_Depth[256];	// Variable zum lokalen Ablegen der Tiefe des aktuellen Knotens

	strcpy(temp_Depth, pt_Depth);	// Abspeichern der Baumtiefe in der eben erzeugten lokalen Variable
	
	for (run_a = 0; run_a < 2; run_a++)	// Schleife, die rekursiv in die Kinderknoten "absteigt", falls diese vorhanden sind
	{
		char temp_Buffer[2];	// Speichervariable den ASCII Wert des Kindknotens ("0" / "1")
		char temp_Buffer2[256];		// Speichervariable die die aktuelle Baumtiefe mit der des Kindknotens verknpft (wird bentigt, da strcat() auch die bergebene Variable verndert)
		strcpy(temp_Buffer2, temp_Depth);	// Kopieren der aktuellen Baumtiefe in den Buffer

		if(run_a == 0)
			(*ManipulateNode0) (pt_RootNode, temp_Depth, pt_ExtraData, pt_file);	// Fr jeden Knoten wird die ber Parameter bergebene Funktion aufgerufen
		else
			(*ManipulateNode1) (pt_RootNode, temp_Depth, pt_ExtraData, pt_file);	// Fr jeden Knoten wird die ber Parameter bergebene Funktion aufgerufen

		(void) itoa(run_a, temp_Buffer, 10);	// Speichern von "0" / "1" abhngig vom Kindknoten der "aufgerufen" wird

		if (pt_RootNode->pt_Down[run_a] != NULL)		// Falls es einen Kindknoten gibt ...
			TraverseTreeforRead(pt_RootNode->pt_Down[run_a], strcat(temp_Buffer2, temp_Buffer), (*ManipulateNode0), (*ManipulateNode1), pt_ExtraData, pt_file); // ... soll rekursiv in diesen "eingesprungen" werden und die Parameter werden weiterbergeben
	}
}

/**
 *
 *	@function: Hngt einen Knoten abhngig von den Daten einer bergebenen Datei an den aktuellen Knoten an und fgt ihn in die Baumstruktur ein
 *	@params:
 *				pt_ActualNode ... Zeiger auf den zu bearbeitenden Knoten
 *				pt_Depth ... (eig. unntig beim lschen der Knoten) Indikator fr die Tiefe des Knotens in der Baumstruktur ("0" entspricht "links abbiegen" in der Baumstruktur und "1" rechts)
 *				pt_ExtraData ... Speichert die BufferBitSequence, um Bitweise aus einer Datei auslesen zu knnen
 *				pt_file ... Datei aus der gelesen wird
 *	@return value:
 *				KEINER
 *
 */
void ReadLeftChildFromFile (Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file)
{
	Node *pt_NewNode = NULL; // Zeiger auf den neu zu erzeugenden Knoten
	char temp_SimpleNode;
	if (pt_ActualNode->iSimpleNode == LEAFNODE)	// Wenn es sich beim Vaterknoten schon um einen Blattknoten handelt, soll nichts angehngt werden
		return;

	temp_SimpleNode = GetBitFromFile(pt_file, (BitSequence *)pt_ExtraData);	// Auslesen, ob es sich beim neuen Knoten um einen Blatt oder Vaterknoten handelt

	if (temp_SimpleNode != -2)
	{
		int cContent = (temp_SimpleNode == LEAFNODE ? GetByteFromFile(pt_file, (BitSequence *)pt_ExtraData) : '\0');			// Falls es sich um einen Blattknoten handelt, so entscheidet das nchste Byte den Inhalt, ansonsten wird '\0' gesetzt
		// Neuer Knoten wird erzeugt
		pt_NewNode = CreateNode( temp_SimpleNode,// iSimpleNode wird auf den eingelesenen Wert aus der Datei gesetzt
								cContent,
								0.,				// Wahrscheinlichkeit auf 0, da fr Dekomprimierung nicht von Interesse
								pt_ActualNode,	// Knoten der vom Traversieren bergeben wird ist Vaterknoten
								NULL,			// Verknpfung zum linken Kindknoten (Beim Auslesen des ersten Knotens noch unbekannt)
								NULL,			// Verknpfung zum rechten Kindknoten (Beim Auslesen des ersten Knotens noch unbekannt)
								NULL);			// Da sich ein ausgelesenerKnoten in keiner Liste befinden soll, wird kein Link erstellt
		// Einhngen des neu erzeugten Knotens
		pt_ActualNode->pt_Down[0] = pt_NewNode;
	}

}
/**
 *
 *	@function: Hngt einen Knoten abhngig von den Daten einer bergebenen Datei an den aktuellen Knoten an und fgt ihn in die Baumstruktur ein
 *	@params:
 *				pt_ActualNode ... Zeiger auf den zu bearbeitenden Knoten
 *				pt_Depth ... (eig. unntig beim lschen der Knoten) Indikator fr die Tiefe des Knotens in der Baumstruktur ("0" entspricht "links abbiegen" in der Baumstruktur und "1" rechts)
 *				pt_ExtraData ... Speichert die BufferBitSequence, um Bitweise aus einer Datei auslesen zu knnen
 *				pt_file ... Datei aus der gelesen wird
 *	@return value:
 *				KEINER
 *
 */
void ReadRightChildFromFile (Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file)
{
	Node *pt_NewNode = NULL; // Zeiger auf den neu zu erzeugenden Knoten
	char temp_SimpleNode;
	if (pt_ActualNode->iSimpleNode == LEAFNODE)	// Wenn es sich beim Vaterknoten schon um einen Blattknoten handelt, soll nichts angehngt werden
		return;

	temp_SimpleNode = GetBitFromFile(pt_file, (BitSequence *)pt_ExtraData);	// Auslesen, ob es sich beim neuen Knoten um einen Blatt oder Vaterknoten handelt

	if (temp_SimpleNode != -2)
	{
		int cContent = (temp_SimpleNode == LEAFNODE ? GetByteFromFile(pt_file, (BitSequence *)pt_ExtraData) : '\0');			// Falls es sich um einen Blattknoten handelt, so entscheidet das nchste Byte den Inhalt, ansonsten wird '\0' gesetzt
		// Erster Knoten wird erzeugt
		pt_NewNode = CreateNode( temp_SimpleNode,// iSimpleNode wird auf den eingelesenen Wert aus der Datei gesetzt
								cContent,
								0.,				// Wahrscheinlichkeit auf 0, da fr Dekomprimierung nicht von Interesse
								pt_ActualNode,	// Knoten der vom Traversieren bergeben wird ist Vaterknoten
								NULL,			// Verknpfung zum linken Kindknoten (Beim Auslesen des ersten Knotens noch unbekannt)
								NULL,			// Verknpfung zum rechten Kindknoten (Beim Auslesen des ersten Knotens noch unbekannt)
								NULL);			// Da sich ein ausgelesenerKnoten in keiner Liste befinden soll, wird kein Link erstellt
		// Einhngen des neu erzeugten Knotens
		pt_ActualNode->pt_Down[1] = pt_NewNode;
	}

}
