//*****************************************************//
//Dateiname: start_and_control_coding.c
//Autor: Walter Scherer
//Projekt: niCE - nice insignificant compression engine
//Zweck: Startet und kontrolliert den Ablauf der Codierung
//Datum der letzten Aenderung: 21.05.2003
//*****************************************************//

#include "../nice.h"
#include "../huffman.h"
#include "../globals.h"

//***************************************************************************
//
//					Funktionen
//
//***************************************************************************
/** 
 * 
 *	@function: Kontrolliert die Codierung und ruft die zur Codierung notwendigen Funktionen auf
 *	@params: 
 *			file_source ... Quelldatei
 *			file_target ... Zieldatei
 *	@return value: 
 *			KEINER
 *
 * 
 */ 
void control_coding(char *file_source, char *file_target)
{
	double *pt_array_probabilities;						//Wahrscheinlichkeitsarray (Zeichenauftrittswahrscheinlichkeiten)
	Node *pt_root_node;									//Root-Knoten
	BitSequence *pt_array_codetable;					//Anfang der Codiertabelle

	printf("analysing %s...\n", file_source);
	pt_array_probabilities=huffman_analyse_program(file_source);		//Huffman-Analyse
	if (pt_array_probabilities==NULL)
	{
		printf("\nnice:coding_control: An error occured while trying to analyse file for huffman code.\n");
		temp_num_error++;
		return;
	}
	printf("creating huffman tree...\n");	
	pt_root_node=CreateTree(pt_array_probabilities);					//Baumerstellung
	if (pt_root_node==NULL) 
	{
		temp_num_error++;
		printf("\nnice:coding_control: an error occured while trying to build huffman tree.\n");
		return;
	}
	printf("creating coding table...\n");
	pt_array_codetable=CreateCodingTable(pt_root_node);					//Codetabelle erstellen
	if (pt_array_codetable==NULL)										//Falls etwas bei der Erstellung der Codetabelle schief luft
	{
		printf("\nnice:coding_control: could not establish code table.\n");
		temp_num_error++;
		return;
	}
	//calc_avg_length_of_code(pt_array_codetable);						//durchschnittliche Codelnge errechnen
	printf("saving data to %s...\n", file_target);
	OpenFiletoSave(file_source, file_target, pt_root_node, pt_array_codetable, 1);			//Speichern des Huffman-Baums und Komprimierung der Datei
	printf("completing algorithm...\n\n");
	DestroyTree(pt_root_node);											//Baum zerstren
	DestroyCodingTable (pt_array_codetable);							//Codetabelle zerstren
}