//*****************************************************//
//Dateiname: statistics.c
//Autor: Walter Scherer
//Projekt: niCE - nice insignificant compression engine
//Zweck: Statistik-Datei, implementiert einige Funktionen zur Bestimmung von Dateistatistiken
//Datum der letzten Aenderung: 21.05.2003
//*****************************************************//

#include "../nice.h"
#include "../huffman.h"
#include "../globals.h"

//***************************************************************************
//
//					Funktionen
//
//***************************************************************************

/** 
 *	@function: Berechnet die Coderate
 *	@params: 
 *			size_source... Gre der Quelldatei
 *			size_target... Gre der Zieldatei
 *			
 *	@return value: 
 *			KEINER
 */

void calc_coding_percentage(double size_source, double size_target)
{
	double coding_percentage=0.;

	coding_percentage=((100*size_target)/size_source);
	printf("\tcode rate: \t\t\t%lf%%\n", coding_percentage);
}

/** 
 *	@function: Berechnet die durchschnittliche Codelnge pro Zeichen
 *	@params: 
 *			pt_array_codetable
 *			
 *	@return value: 
 *			KEINER
 */

void calc_avg_length_of_code(BitSequence *pt_array_codetable)
{
	int run_a;
	float avg_code_length=0.;
	int counter=0;
	int num_crucial_chars=0;
	for (run_a=0; run_a<256; run_a++)
	{
		if ((pt_array_codetable[run_a].length)!=0)
		{
			num_crucial_chars++;
			counter=counter+((int)(pt_array_codetable[run_a].length));
		}
	}
	avg_code_length=((float)counter/(float)num_crucial_chars);
	printf("\taverage length of code: \t\t\t%lf\n", avg_code_length);
}

/** 
 *	@function: Startet die Statistikfunktion
 *	@params: 
 *			filename ... Quelldatei
 *			
 *	@return value: 
 *			KEINER
 */
double start_general_statistics(char *filename)
{
	int num_chars;
	double size_file;
	printf("-------------------------------------------------------------\n");
	printf("\t\tfile statistics: file %s\n", filename);
	printf("-------------------------------------------------------------\n");
	num_chars=get_number_of_chars(filename);
	size_file=get_size_of_file(filename, num_chars);
	printf("-------------------------------------------------------------\n");
	return size_file;
}

/** 
 *	@function: Zhlt die Zeichen in der Datei (bestimmt also die Gre) (findet Gre ber direktes Zeichenzhlen in der Datei heraus)
 *	@params: 
 *			filename ... Quelldatei
 *			
 *	@return value: 
 *			Characteranzahl
 */
int get_number_of_chars(char *filename)
{
	FILE *pt_file;
	char temp_char;
	int temp_number=0;
	if ((pt_file=open_file_read_access(filename))==NULL)
	{
		printf("an error occurred when trying to retrieve file statistics.\n");
		return -1;
	}
	while ((temp_char=getc(pt_file)!=EOF))
		temp_number++;
	fclose(pt_file);
	printf("\ttotal number of characters:\t\t\t%d\n", temp_number);
	return temp_number;
}

/** 
 *	@function: Gibt die Gre der bergebenen Datei aus (findet Gre ber Inode heraus)
 *	@params: 
 *			filename ... Quelldatei
 *			num_chars ... Zeichenanzahl
 *			
 *	@return value: 
 *			stbuf.st_size ... Dateigre
 */
double get_size_of_file(char *filename, int num_chars)
{
	struct stat stbuf;							//Inode-Informationen ber die Datei

	if (stat(filename, &stbuf) == -1)			//Holen der Inode-Informationen der Datei
	{
		fprintf(stderr, "nice: cannot access %s\n", filename);
		return -1;
	}
	printf("\tsize of file: \t\t\t\t\t%ld Bytes\n", stbuf.st_size);
	calc_entropy(filename, num_chars);
	return stbuf.st_size;
}

/** 
 *	@function: Berechnet den durchschnittlichen Informationsgehalt der Zeichen
 *	@params: 
 *			filename ... Quelldatei
 *			num_chars ... Zeichenanzahl
 *			
 *	@return value: 
 *			KEINER
 */
void calc_entropy(char *filename, int num_chars)
{
	int run_a;
	int num_crucial_chars=0;									//Anzahl der vorkommenden Zeichen (der Zeichen, bei denen der Eintrag im Wahrscheinlichkeitsarray nicht gleich 0. ist
	double *pt_array_probability;
	double entropy=0.;
	pt_array_probability=huffman_analyse_program(filename);		//Wahrscheinlichkeitsarray mit Rckgabewert der Huffman-Analyse-Funktion belegen
	for (run_a=0;run_a<256;run_a++)
	{
		if (pt_array_probability[run_a]!=0.) 
		{
			num_crucial_chars++;
			pt_array_probability[run_a]=(pt_array_probability[run_a]*((log(1/pt_array_probability[run_a]))/log(2.)));	//Berechnung der Entropie (dafr entweder Fachliteratur zu Rate ziehen oder in der Doku nachsehen)
			entropy=entropy+pt_array_probability[run_a];				//Einzelne Entropiewerte aufsummieren
		}
	}
	printf("\tentropy: \t\t\t\t\t%.10lf Bit\n", entropy);		//Ausgabe der Entropie auf 10 Nachkommastellen genau
	printf("\tnumber of different characters: \t\t%d\n", num_crucial_chars);	//Anzahl der unterschiedlichen Zeichen ausgeben (Nebenprodukt dieser Funktion, ist aber auch interessant)
}

/** 
 *	@function: ffnet eine Datei zum Lesen
 *	@params: 
 *			filename ... Quelldatei
 *
 *			
 *	@return value: 
 *			Filepointer
 */
FILE *open_file_read_access(char *filename)
{
	FILE *pt_file;
	if ((pt_file=(fopen(filename, "rb")))==NULL)
	{
		printf("nice: could not open file.\n");
		temp_num_error++;
		return NULL;
	}
	return pt_file;
}