//*****************************************************//
//Dateiname: huffman.h
//Autor: Stephan Ptschner, Walter Scherer
//Projekt: niCE - nice insignificant compression engine
//Zweck: Header-Datei fr Operationen mit Huffman (Codierung, Decodierung)
//Datum der letzten Aenderung: 21.05.2003
//*****************************************************//

#ifndef HUFFMAN_FUNCTIONS
	#define HUFFMAN_FUNCTIONS
	#define CHAR_LENGTH 1			//Lnge des nach der Codefolge in die Datei zu schreibende Zeichen
	#define INT_LENGTH 1			//Lnge des als Codefolge pro Durchgang zu schreibender Integer
	#define LEFT 0
	#define RIGHT 1
	#define REQUESTED_EXTENSION "nice"	//Bezeichnung der gewnschten Dateierweiterung fr alle Zieldateien bei Komprimierung (immer in "" setzen)
	#define LENGTH_OF_EXTENSION 4		//Lnge der obigen Erweiterung (zur berprfung)
	#define LEAFNODE 1
	#define ROOTNODE 0

// Strukturen
typedef struct _Node
{
	// Inhaltselemente
	unsigned char cContent;				// Falls iSimpleNode = 0: Buchstaben-Inhalt des Blattes; ansonsten -> '\0'
	double ldProbability;	// Wahrscheinlichkeit des Auftretens der Knoten: Blattknoten: Auftrittsswahrscheinlichkeit von cContent; ansonsten Summe aller "Kinderwahrscheinlichkeiten"
	
	// Baumelemente
	int iSimpleNode;			// Handelt es sich um einen Blattknoten mit Buchstabeninfo -> 1, ansonsten -> 0
	struct _Node *pt_Up;		// Verknpfung zum Vaterknoten
	struct _Node *pt_Down[2];	// Verknpfung zu den 2 Kindknoten
	
	// Listenelemente
	struct _Node *pt_Next;		// Verknpfung zum Listennachbarn
} Node;

typedef struct
{
	unsigned int *Code;
	unsigned char length;
} BitSequence;

//***************************************************************************
//
//					FunktionsPROTOTYPEN
//
//***************************************************************************

	/**
	 *
	 * start_and_control_coding.c - Funktionen zur Kontrolle der Codierung
	 *
	 */
void control_coding(char *file_source, char *file_target);

	/**
	 *
	 * huffman_analyse.c - Funktionen zur Analyse einer RLE- oder vom User spezifizierten Datei
	 *
	 */
void calc_probability(int temp_counter, double *pt_array_probability);
void count_char(int temp_character, double *pt_array_probability);
double *huffman_analyse_program(char *file);

	/**
	 *
	 * CreateTree.c - Funktionen zum Erzeugen eines dynamischen Baumes und einer Liste
	 *
	 */
Node *CreateTree(double *pt_array_Probabilities);
Node *CreateTreeFromFile(FILE *pt_File, BitSequence *pt_BufferSequence);
Node *CreateList (double *pt_array_Probabilities);
Node *CreateNode (int iSimpleNode, char cContent, double ldProbability, Node *pt_Up, Node *pt_DownLeft, Node *pt_DownRight, Node *pt_Next);
Node *InsertNode (Node *pt_BeginOfList, Node *pt_InsertNew);
void DestroyTree(Node *pt_rootNode);


	/**
	 *
	 * TraverseTree.c - Funktionen zum Erzeugen eines dynamischen Baumes und einer Liste
	 *
	 */
void TraverseTree(Node *pt_RootNode, char *Depth, void (*ManipulateNode)(Node *pt_ActualNode, char *Depth, void *pt_ExtraData), void *pt_ExtraData);
void TraverseTreePreorder(Node *pt_RootNode, char *pt_Depth, void (*ManipulateNode)(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData), void *pt_ExtraData);
void TraverseTreeforSave(Node *pt_RootNode, char *pt_Depth, void (*ManipulateNode)(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file), void *pt_ExtraData, FILE *pt_file);
void TraverseTreeforRead(Node *pt_RootNode, char *pt_Depth, void (*ManipulateNode0)(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file) , void (*ManipulateNode1)(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file), void *pt_ExtraData, FILE *pt_file);

void ReadLeftChildFromFile (Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file);
void ReadRightChildFromFile (Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file);
void DeleteNode(Node *pt_ActualNode, char *Depth, void *pt_ExtraData);
void AddNodeToCodingTable(Node *pt_ActualNode, char *Depth, void *pt_ExtraData);
void SaveNodetoFile(Node *pt_ActualNode, char *pt_Depth, void *pt_ExtraData, FILE *pt_file);

	/**
	 *
	 * BitOperations.c - Funktionen zum Erzeugen eines dynamischen Baumes und einer Liste
	 *
	 */
#define MEMSIZE(BitLength) ((BitLength) % (sizeof(int) * 8) == 0 ? (char)((BitLength) / (sizeof(int) * 8)) : (char)((BitLength) / (sizeof(int) * 8) + 1))

int CreateBitSequence (char *pt_String, BitSequence *pt_GeneratedCode);
double PowerOf (long base, int exponent);
int IsBitSet (int CheckSequence, char CheckBitNr);
void ShiftLeft(BitSequence *ActualBitSequence);
int IsBitSetOfBS (BitSequence CheckSequence, unsigned char CheckBitNr);

	/**
	 *
	 * CreateCodingTable.c - Funktionen zum Erzeugen eines dynamischen Baumes und einer Liste
	 *
	 */
BitSequence *CreateCodingTable (Node *pt_RootNode);
void DestroyCodingTable (BitSequence *pt_array_CodingTable);

	/**
	 *
	 * CompressFile.c - Funktionen zum Erzeugen eines dynamischen Baumes und einer Liste
	 *
	 */
void CompressFile (FILE *pt_FileUncompressed, FILE *pt_FileCompressed, BitSequence *pt_array_CodingTable, BitSequence BufferBitSequence, char Continue);
int DecompressFile (char *file_source, char *file_target);

	/**
	 *
	 * FileOperations.c - Funktionen zum Erzeugen eines dynamischen Baumes und einer Liste
	 *
	 */
int WriteBitSequenceToFile(FILE *pt_file, BitSequence *pt_BufferBitSequence, BitSequence *pt_ActualBitSequence);
char GetBitFromFile(FILE *pt_File, BitSequence *pt_BufferSequence);
char GetByteFromFile(FILE *pt_File, BitSequence *pt_BufferSequence);
int GetBitsequenceFromFile(FILE *pt_File, BitSequence *pt_BufferSequence);

	/**
	 *
	 * save_tree.c - Funktionen zum Speichern des Baums in die Komprimierdatei
	 *
	 */
BitSequence SaveTree(FILE *pt_file, Node *pt_RootNode, BitSequence BufferBitSequence);
void OpenFiletoSave(char *file_name_source, char *file_name_target, Node *pt_RootNode, BitSequence *pt_array_codetable, char Continue);
//***************************************************************************
#endif

