//*****************************************************//
//Dateiname: nice_main.c
//Autor: Walter Scherer
//Projekt: niCE - nice insignificant compression engine
//Zweck: Parametereingabe von Userseite
//Datum der letzten Aenderung: 21.05.2003
//*****************************************************//

int temp_num_error=0;

#include "nice.h"
#include "huffman.h"
#include "parameter.h"
#include "globals.h"
#include "rle.h"

//***************************************************************************
//
//					Funktionen
//
//***************************************************************************
/** 
 * 
 *	@function: berprft, ob die Dateierweiterung bei der Zieldatei richtig ist (Erweiterung muss .nice sein)
 *	@params:			filename ... Dateiname der Zieldatei
 *	@return value:		Fehlerwert (1 bei Fehler, 0 bei ok)
 *
 */
int check_target_extension(char *filename)
{
	int run_a;
	int run_b=0;
	int start_position_extension=0;					//Position des letzten Punktes im Dateinamen
	int int_check_extension=0;						//Bei 1 Fehler, bei 0 ok
	char *check_extension=REQUESTED_EXTENSION;		//REQUESTED_EXTENSION definiert in huffman.h

	for (run_a=0;run_a<(int)strlen(filename);run_a++)
	{
		if ((filename[run_a])=='.')					//Bei jedem '.' im Dateinamen die Position speichern
		{
			start_position_extension=run_a+1;		//da run_a bei 0 beginnt, muss die Positionsvariable um 1 hher angesetzt werden
		}
	}												//hier steht die Position des letzten '.' im Dateinamen fest
	if ((strcmp(filename+start_position_extension, check_extension))!=0)		//Vergleiche den Dateinamen ab dem Punkt mit der gewnschten Erweiterung, bei 0 herrscht Gleichheit
	{
		printf("extension of %s should be \"nice\".\n", filename);
		temp_num_error++;
		return 1;									//Fehler (der Dateiname ist nicht gleich REQUESTED_EXTENSION mit Lnge LENGTH_OF_EXTENSION
	}
	return 0;
}

/** 
 * 
 *	@function: Gibt Benutzungshinweise wieder
 *	@params: none
 *	@return value: void
 *
 */
void usage() 
{
	printf("\n\nUsage: nice -[option1] -[option2] -[option3] <file(s)>(order is dc (don't care))\n");
	printf("   or: nice -[options] (order is dc (don't care))\n");
	printf("\n");
	printf("Options:\n");
	printf("\tc\texecutes coding algorithm with specified file\n");
	printf("\t\tand compresses source file. source file must have \"nice\" \n\t\tas extension.\n");
	printf("\t\tattempts to write compressed file (file name is second file).\n");
	printf("\t\tnote: overwrites target file if existent.\n\n");
	printf("\td\tdecodes source file (first argument) and writes decoded file\n\t\tinto target file (second argument).\n\t\tnote: overwrites target file if existent.\n\n");
	printf("\ts\tlists statistics of the specified file(s)\n\n");
}
/** 
 * 
 *	@function: Liest Parameter ein und berprft diese, Startet Funktionsaufrufe
 *	@params: Integer-ArgumentCounter, Characterpointer auf das Argumentarray
 *	@return value: Integerberprfungsrckgabewert
 *
 */
int check_parameters_main (int argc, char *argv[]) 
{
	char *file_source;								//Char-Pointer auf den Dateinamen der Quelldateien
	char *file_target;								//Char-Pointer auf den Dateinamen der Zieldatei
	char *temp_filename;							//wird bentigt um RLE-Datei zwischenzuspeichern
	double size_source;								//Gre der Quelldatei
	double size_target;								//Gre der Zieldatei

	int temp_check_valid=0;							//berprfung, ob korrekte Parameterangabe erfolgt ist (0 bedeutet falsch)
	int temp_get_parameter;							//bezeichnet den gerade zu bearbeitenden Parameter (c, d oder s)

	while ((--argc > 0) && (**++argv == '-'))		//1.Bedingung: der um 1 reduzierte argc (argument counter) muss ber 0 liegen (sonst ist kein Parameter da)
	{												//2.Bedingung: Betrachte jedes als Parameter eingegebene Zeichen und finde heraus, wo ein Parameterzeichen (ein "Minuszeichen") steht, erst dann innerhalb der while-Schleife weitermachen (Nheres dazu im Manual, bzw. Doku)
		temp_check_valid = 1;						//Korrekte Parametereingabe (temp_check_valid ist 1)
		while (temp_get_parameter = *++argv[0])		//Der argv (argument vector) steht zu diesem Zeitpunkt ein Zeichen vor dem auszufhrenden Parameter, daher erhhe ihn um 1, jetzt steht er auf der Position des Parameters
		{											//Arbeitsweise gut im Debug-Modus erkennbar (argv berwachen!)
			switch (temp_get_parameter) 
			{
			case 'c':										//Komprimierung einer Datei
				file_source=argv[1];
				file_target=argv[2];
				if ((file_source==NULL) || (file_target==NULL))
				{
					printf("an parameter error occurred. you probably forgot to specify the files.\n");
					temp_num_error++;
					break;
				}
				if (check_target_extension(file_target)==1)	//Wenn die Erweiterung der Zieldatei nicht den Konventionen entspricht
					break;
				if ((file_source!=NULL)&&(file_target!=NULL))
				{
					printf("coding procedure in progress. source: %s, target: %s.\n\nplease wait...\n", file_source, file_target);
					printf("rle - coding procedure in progress. ");
					file_source = rle_cod(file_source);
					printf("huffman - coding procedure in progress. ");
					control_coding(file_source, file_target);
					remove(file_source);
				}
				else
				{
					printf("You have to specify the source file and then the target file after parameter '-c'\n");
					temp_num_error++;
				}
				break;
			case 'd':											//Dekomprimierung einer Datei
				file_source=argv[1];
				file_target=argv[2];

				if( (temp_filename = _tempnam("c:\\", "nice_tmp" ) ) == NULL )
				{
					printf( "Cannot create a unique filename\n" );
					return 1;
				}

				if ((file_source != NULL) && (file_target != NULL) && (temp_filename != NULL))
				{
					printf("decoding procedure in progress. please wait...\n");
					printf("decoding huffman-encoded data. please wait...\n");
					if (DecompressFile(file_source, temp_filename))
						printf("You have to specify the source file and then the target file after parameter '-c'\n");
					else
					{
						printf("decoding rle-encoded data. please wait...\n");
						rle_dcod(temp_filename, file_target);
						printf("decoding procedure finished.\n");
						remove(temp_filename);
					}
				}
				else
				{
					printf("You have to specify source file and then target file after parameter '-h'\n");
					temp_num_error++;
				}
				break;
			case 's':
				file_source=argv[1];
				if (file_source==NULL)
				{
					printf("please specify at least one file after parameter '-s'\n");
					temp_num_error++;
					break;
				}
				size_source=start_general_statistics(file_source);
				if ((file_target=argv[2])!=NULL)
				{
					size_target=start_general_statistics(file_target);
					if (size_target>size_source)
						printf("WARNING: target file is bigger than source file.\n");
					calc_coding_percentage(size_source, size_target);
				}
				break;
			default:
				printf("nice: illegal option %c\n", temp_get_parameter);
				temp_num_error++;
				break;
			}
		}
	}
	if (temp_check_valid == 0) 
	{
		usage();
		return 1;
	}
	switch (temp_num_error) 
	{
	case 0:
		printf("\nprogram finished.\n");
		return 0;
		break;
	case 1:
		printf("\n%d error occurred. Programm terminated.\n", temp_num_error);
		usage();
		return 1;
		break;
	default:
		printf("\n%d errors occurred. Programm terminated.\n", temp_num_error);
		usage();
		return 1;
	}
}
/** 
 * 
 *	@function: Liest Parameter ein und ruft berprfungsfunktion auf
 *	@params: Integer-ArgumentCounter, Characterpointer auf das Argumentarray
 *	@return value: void
 *
 */
void main (int argc, char *argv[]) 
{
	int temp_check_parameters = check_parameters_main(argc,argv);
}